﻿using System;

namespace TestProject
{
    /// <summary>
    /// BASE64 URL 인코딩 헬퍼
    /// </summary>
    public static class Base64URLEncodingHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 인코딩하기 - Encode(sourceByteArray)

        /// <summary>
        /// 인코딩하기
        /// </summary>
        /// <param name="sourceByteArray">소스 바이트 배열</param>
        /// <returns>BASE64 문자열</returns>
        public static string Encode(byte[] sourceByteArray)
        {
            if(sourceByteArray == null)
            {
                throw new ArgumentNullException("sourceByteArray");
            }

            return Convert.ToBase64String(sourceByteArray).TrimEnd('=').Replace('+', '-').Replace('/', '_');
        }

        #endregion
        #region 디코딩하기 - Decode(source)

        /// <summary>
        /// 디코딩하기
        /// </summary>
        /// <param name="source">BASE64 문자열</param>
        /// <returns>바이트 배열</returns>
        public static byte[] Decode(string source)
        {
            if(source == null)
            {
                throw new ArgumentNullException("source");
            }

            return Convert.FromBase64String(SetBase64Padding(source.Replace('-', '+').Replace('_', '/')));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region BASE64 패딩 설정하기 - SetBase64Padding(source)

        /// <summary>
        /// BASE64 패딩 설정하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        /// <returns>BASE64 패딩 적용 문자열</returns>
        private static string SetBase64Padding(string source)
        {
            int padding = 3 - ((source.Length + 3) % 4);

            if(padding == 0)
            {
                return source;
            }

            return source + new string('=', padding);
        }

        #endregion
    }
}